@@ -1,42 +1,61 @@
 # cuerank
 #'   Calculate the accuracy of all cues in a dataframe
 #'
-#' @param cue.df A dataframe of cue values
-#' @param criterion.v A logical vector of length m containing criterion values for exemplars in cue.df
+#' @param formula A formula specifying a binary criterion as a function of multiple variables
+#' @param data A dataframe containing variables in formula
 #' @param tree.criterion A string indicating how to rank cues. "v" = HR - FAR, "d" = d-prime.
 #' @param numthresh.method A string indicating how to calculate cue splitting thresholds. "m" = median split, "o" = split that maximizes the tree criterion.
 #' @param hr.weight A number between 0 and 1 indicating how much weight to give to increasing hit rates versus avoiding false alarms. 1 means maximizing HR and ignoring FAR, while 0 does the opposite. The default of 0.5 gives equal weight to both.
 #' @param rounding An integer indicating digit rounding for non-integer numeric cue thresholds. The default is NULL which means no rounding. A value of 0 rounds all possible thresholds to the nearest integer, 1 rounds to the nearest .1 (etc.).
 #' @param verbose A logical value indicating whether or not to print ongoing diagnostics
+#' @param cue.rules An optional df specifying how to make decisions for each cue. Must contain columns "cue", "class", "threshold" and "direction"
 #' @importFrom stats median
-#' @return A list with two elements. The first, "cue.stats" is a dataframe containing the classification statistics for every possible cue threshold. The second element, cue.best, contains data for the best performing cue threshold
+#' @return A dataframe containing marginal classification statistics for each cue
 #' @export
 #'
 
 
-cuerank <- function(cue.df = NULL,
-                    criterion.v = NULL,
+cuerank <- function(formula = NULL,
+                    data = NULL,
                     tree.criterion = "v",
                     numthresh.method = "o",
                     hr.weight = .5,
                     rounding = NULL,
-                    verbose = F
+                    verbose = F,
+                    cue.rules = NULL
 
 ) {
 
 
-
-# -----
 # TESTING GROUNDS
 # -----
 
-  # cue.df <- heartdisease[,1:5]
+  # formula <- diagnosis ~.
+  # data <- heartdisease[1:75,]
+  #
+  # cue.df <- heartdisease
   # criterion.v <- heartdisease$diagnosis
   # tree.criterion <- "v"
   # numthresh.method <- "o"
   # hr.weight <- .5
   # correction <- .25
-  # rounding <- NULL
+  # rounding <- 2
+  # cue.rules <- NULL
+
+
+  # EXTRACT FORMULA VARIABLES
+
+  data.mf <- model.frame(formula = formula, data = data)
+  criterion.v <- data.mf[,1]
+  cue.df <- data.mf[,2:ncol(data.mf)]
+
+  if(class(cue.df) != "data.frame") {
+
+    cue.df <- data.frame(cue.df)
+    names(cue.df) <- names(data.mf)[2]
+
+  }
+
   #
   #
   #
@@ -48,20 +67,32 @@ cuerank <- function(cue.df = NULL,
   #
 
 
-  correction <- .25
+# GLOBAL VARIABLES (could be updated later)
 
+correction <- .25
 max.numcat <- 20
 n.cues <- ncol(cue.df)
 
+# CHECK cue.rules
+
+if(is.null(cue.rules) == F) {
+
+if(all(c("cue", "class", "threshold", "direction") %in% names(cue.rules)) == F) {
+
+  stop("You specified a cue.rules object but the object does not contain critical columns. Look at the descriptions for cue.rules and try again")
+
+}
+}
+
 for(cue.i in 1:n.cues) {
 
-  cue.name <- names(cue.df)[cue.i]
-  cue.class <- class(cue.df[,cue.i])
+  cue <- names(cue.df)[cue.i]
+  class <- class(cue.df[,cue.i])
   cue.v <- unlist(cue.df[,cue.i])
 
-  if(setequal(cue.class, c("ordered", "factor"))) {
+  if(setequal(class, c("ordered", "factor"))) {
 
-    cue.class <- "integer"
+    class <- "integer"
     cue.v <- as.numeric(cue.v)
 
   }
@@ -70,9 +101,9 @@ for(cue.i in 1:n.cues) {
 
   # Step 0B: Determine cue levels
 
-  if(cue.class %in% c("numeric", "integer")) {
+  if(class %in% c("numeric", "integer")) {
 
-    if(numthresh.method == "o") {
+    if(numthresh.method == "o" & is.null(cue.rules)) {
 
       cue.levels <- sort(unique(unlist(cue.v)))
 
@@ -84,38 +115,64 @@ for(cue.i in 1:n.cues) {
 
     }
 
-    if(numthresh.method == "m") {
+    if(numthresh.method == "m" & is.null(cue.rules)) {
 
       cue.levels <- median(unlist(cue.v))
 
     }
 
+    if(is.null(cue.rules) == F) {
+
+      cue.levels <- as.numeric(cue.rules$threshold[cue.rules$cue == cue])
+
+    }
+
+
     if(!is.null(rounding)) {cue.levels <- round(cue.levels, digits = rounding)}
 
     cue.levels <- cue.levels[duplicated(cue.levels) == FALSE]
 
   }
 
-  if((cue.class %in% c("numeric", "integer")) == FALSE) {
+  if((class %in% c("numeric", "integer")) == FALSE) {
+
+    if(is.null(cue.rules)) {
 
       cue.levels <- sort(unique(unlist(cue.v)))
 
+    }
+
+    if(is.null(cue.rules) == F) {
+
+      cue.levels <- cue.rules$threshold[cue.rules$cue == cue]
+
+    }
+
   }
 
 
   cue.n <- length(cue.levels)
   accuracy.names <- names(classtable(c(1, 0, 1), c(1, 1, 0)))
 
-
-  # Step 1A: Determine best sigdirection for all cue levels
+  # Step 1A: Determine best direction for all cue levels
 
   # FACTORS
 
-  if(cue.class %in% c("factor", "character", "logical")) {
+  if(class %in% c("factor", "character", "logical")) {
 
-    level.sigdirection.vec <- c("=", "!=")
+    if(is.null(cue.rules)) {
 
-    for(level.sigdirection.i in level.sigdirection.vec) {
+    direction.vec <- c("=", "!=")
+
+    }
+
+    if(is.null(cue.rules) == F) {
+
+    direction.vec <- cue.rules$direction[cue.rules$cue == cue]
+
+    }
+
+    for(direction.i in direction.vec) {
 
       # cue.stats.o
       # Get accuracy of all individual cue levels
@@ -124,12 +181,12 @@ for(cue.i in 1:n.cues) {
         cue.stats.o <- as.data.frame(matrix(NA, nrow = length(cue.levels),
                                             ncol = 4))
 
-        names(cue.stats.o) <- c("cue.name", "cue.class", "level.threshold", "level.sigdirection")
+        names(cue.stats.o) <- c("cue", "class", "threshold", "direction")
 
-        cue.stats.o$cue.name <- cue.name
-        cue.stats.o$cue.class <- cue.class
-        cue.stats.o$level.threshold <- cue.levels
-        cue.stats.o$level.sigdirection <- level.sigdirection.i
+        cue.stats.o$cue <- cue
+        cue.stats.o$class <- class
+        cue.stats.o$threshold <- cue.levels
+        cue.stats.o$direction <- direction.i
 
         cue.stats.o[accuracy.names] <- NA
 
@@ -137,14 +194,14 @@ for(cue.i in 1:n.cues) {
 
         for(cue.level.i in cue.levels) {
 
-          if(level.sigdirection.i == "=") {pred.vec <- cue.v == cue.level.i}
-          if(level.sigdirection.i == "!=") {pred.vec <- cue.v != cue.level.i}
+          if(direction.i == "=") {pred.vec <- cue.v == cue.level.i}
+          if(direction.i == "!=") {pred.vec <- cue.v != cue.level.i}
 
           classtable.temp <- classtable(prediction.v = pred.vec,
                                         criterion.v = criterion.v)
 
 
-          cue.stats.o[cue.stats.o$level.threshold == cue.level.i, names(classtable.temp)] <- classtable.temp
+          cue.stats.o[cue.stats.o$threshold == cue.level.i, names(classtable.temp)] <- classtable.temp
 
         }
 
@@ -162,23 +219,23 @@ for(cue.i in 1:n.cues) {
       cue.stats <- as.data.frame(matrix(NA, nrow = length(cue.levels),
                                         ncol = 4))
 
-      names(cue.stats) <- c("cue.name", "cue.class", "level.threshold", "level.sigdirection")
-      cue.stats$cue.name <- cue.name
-      cue.stats$cue.class <- cue.class
-      cue.stats$level.sigdirection <- level.sigdirection.i
+      names(cue.stats) <- c("cue", "class", "threshold", "direction")
+      cue.stats$cue <- cue
+      cue.stats$class <- class
+      cue.stats$direction <- direction.i
 
       cue.stats[accuracy.names] <- NA
 
       for (row.index.i in 1:nrow(cue.stats.o)) {
 
-        levels.i <- cue.stats.o$level.threshold[1:row.index.i]
+        levels.i <- cue.stats.o$threshold[1:row.index.i]
 
-        cue.stats$level.threshold[row.index.i] <- paste(levels.i, collapse = ",")
+        cue.stats$threshold[row.index.i] <- paste(levels.i, collapse = ",")
 
         joint.cue.vec <- cue.v %in% levels.i
 
-        if(level.sigdirection.i == "=") {pred.vec <- joint.cue.vec == T}
-        if(level.sigdirection.i == "!=") {pred.vec <- joint.cue.vec == F}
+        if(direction.i == "=") {pred.vec <- joint.cue.vec == T}
+        if(direction.i == "!=") {pred.vec <- joint.cue.vec == F}
 
 
         classtable.temp <- classtable(prediction.v = pred.vec,
@@ -190,34 +247,44 @@ for(cue.i in 1:n.cues) {
 
       }
 
-      if(level.sigdirection.i == level.sigdirection.vec[1]) {cue.stats.all <- cue.stats}
-      if(level.sigdirection.i != level.sigdirection.vec[1]) {cue.stats.all <- rbind(cue.stats.all, cue.stats)}
+      if(direction.i == direction.vec[1]) {cue.stats.all <- cue.stats}
+      if(direction.i != direction.vec[1]) {cue.stats.all <- rbind(cue.stats.all, cue.stats)}
 
       }
     }
 
     cue.stats <- cue.stats.all
 
-
   }
 
   # NUMERIC
 
-  if(cue.class %in% c("numeric", "integer")) {
+  if(class %in% c("numeric", "integer")) {
+
+    if(is.null(cue.rules)) {
+
+    direction.vec <- c("<", "<=", ">", ">=")
+
+    }
+
+    if(is.null(cue.rules) == F) {
+
+    direction.vec <- cue.rules$direction[cue.rules$cue == cue]
+
+    }
 
-    sigdirection.vec <- c("<", "<=", ">", ">=")
 
     cue.stats <- as.data.frame(matrix(NA, nrow = cue.n, ncol = 4))
 
-    names(cue.stats) <- c("cue.name",
-                          "cue.class",
-                          "level.threshold",
-                          "level.sigdirection")
+    names(cue.stats) <- c("cue",
+                          "class",
+                          "threshold",
+                          "direction")
 
     cue.stats[accuracy.names] <- NA
-    cue.stats$cue.name <- cue.name
-    cue.stats$cue.class <- cue.class
-    cue.stats$level.threshold <- cue.levels
+    cue.stats$cue <- cue
+    cue.stats$class <- class
+    cue.stats$threshold <- cue.levels
 
     # Loop over all possible cue levels
 
@@ -225,38 +292,38 @@ for(cue.i in 1:n.cues) {
 
       accuracy.names <- names(classtable(c(1, 1, 0), c(0, 1, 0)))
 
-      sigdirection.accuracy.df <- as.data.frame(matrix(NA,
-                                                       nrow = length(sigdirection.vec),
+      direction.accuracy.df <- as.data.frame(matrix(NA,
+                                                       nrow = length(direction.vec),
                                                        ncol = length(accuracy.names)))
 
-      names(sigdirection.accuracy.df) <- accuracy.names
+      names(direction.accuracy.df) <- accuracy.names
 
-      sigdirection.accuracy.df$level.sigdirection <- sigdirection.vec
+      direction.accuracy.df$direction <- direction.vec
 
       # Loop over 4 directions: <, <=, >, >=
 
-      for(sigdirection.i in sigdirection.vec) {
+      for(direction.i in direction.vec) {
 
-        if(sigdirection.i == "<") {pred.vec <- cue.v < level.i}
-        if(sigdirection.i == "<=") {pred.vec <- cue.v <= level.i}
-        if(sigdirection.i == ">") {pred.vec <- cue.v > level.i}
-        if(sigdirection.i == ">=") {pred.vec <- cue.v >= level.i}
+        if(direction.i == "<") {pred.vec <- cue.v < level.i}
+        if(direction.i == "<=") {pred.vec <- cue.v <= level.i}
+        if(direction.i == ">") {pred.vec <- cue.v > level.i}
+        if(direction.i == ">=") {pred.vec <- cue.v >= level.i}
 
         classtable.temp <- classtable(prediction.v = pred.vec,
                                       criterion.v = criterion.v)
 
-        sigdirection.accuracy.df[which(sigdirection.i == sigdirection.vec), names(classtable.temp)] <- classtable.temp
+        direction.accuracy.df[which(direction.i == direction.vec), names(classtable.temp)] <- classtable.temp
 
       }
 
       # Determine best direction for level.i
 
-      best.acc <- max(sigdirection.accuracy.df[tree.criterion], na.rm = T)
-      best.acc.index <- which(sigdirection.accuracy.df[tree.criterion] == best.acc)
+      best.acc <- max(direction.accuracy.df[tree.criterion], na.rm = T)
+      best.acc.index <- which(direction.accuracy.df[tree.criterion] == best.acc)
 
       if(length(best.acc.index) > 1) {best.acc.index <- sample(best.acc.index, size = 1)}
 
-      best.level.stats <- sigdirection.accuracy.df[best.acc.index,]
+      best.level.stats <- direction.accuracy.df[best.acc.index,]
 
 
       cue.stats[cue.levels == level.i, names(best.level.stats)] <- best.level.stats
@@ -265,7 +332,7 @@ for(cue.i in 1:n.cues) {
 
   }
 
-  if((cue.class %in% c("factor", "character", "logical", "numeric", "integer")) == F) {
+  if((class %in% c("factor", "character", "logical", "numeric", "integer")) == F) {
 
     direction.i <- NA
     v.i <-  NA
